package edu.dartmouth.cs.runonuithread;

import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "runOnUiThread";
    private Button btn_count;
    private Button btn_blank;
    private int i = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        btn_count = findViewById(R.id.btn_count);
        btn_blank = findViewById(R.id.btn_blank);

        Log.d(TAG, "MainThread() Name " + Thread.currentThread().getName()
                + " ID " + Thread.currentThread().getId()
                + " State " + Thread.currentThread().getState());

    }


    public void startThread(View view) {
        runThread();
    }

    // create a thread that counts and to write on the UI thread
    // it coordinates via runOnUiThread()

    private void runThread() {

        new Thread() {
            public void run() {

                Log.d(TAG, "runThread() Name " + this.getName()
                        + " ID " + this.getId()
                        + " State " + this.getState());
                updateButton();
                while (i++ < 1000) {
                    try {
                        runOnUiThread(new Runnable() {

                            @Override
                            public void run() {
                                btn_count.setText("#" + i);
                                Log.d(TAG, "UI btn_count/ Thread Name " + Thread.currentThread().getName()
                                        + " ID " + Thread.currentThread().getId()
                                        + " State " + Thread.currentThread().getState());

                            }
                        });
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }

            // illustrates post(runnable) on a view (button) from a background thread
            // this is cool. post(Runnable action) causes the Runnable to be added
            // to the message queue.

            // notice here we define a unit of execution -- runnable and pass it
            // you hand the work (the runnable) to some other View.post(runnable)
            // in the case the button to handle. Cool!

            private void updateButton() {

                btn_blank.post(new Runnable() {

                    @Override
                    public void run() {
                        Log.d(TAG, "UI btn_blank.post/ Thread Name " + Thread.currentThread().getName()
                                + " ID " + Thread.currentThread().getId()
                                + " State " + Thread.currentThread().getState());
                        btn_blank.setText(R.string.posting_back);
                    }
                });
            }
        }.start();
    }

    // freeze() means you can't interact with any of button after the first click
    // until the delay or freeze is completed. The complete UI is blocked. Try this
    // - click the "Start runOnUIThread to Count"
    // - then click the "start wait thread button"
    // Notice that the UI is blocked and the count not updated. Why is this? Because
    // the UI thread is blocked by a wait and while the runThread() is running
    // the runOnUiThread() runnable can't access the UI's btn_blank.setText until
    // the freeze() completes.
    //
    // nonBlocking() creates a new thread for every click to delay.
    //
    // comment one out an observe the behavior


    public void startWaitThread(View view) {

        // this method will cause the button to block and be
        // no responsive. Try clicking on the button multiple
        // times in one shot -- notice it's blocked ANR
        // Also notice the count is not updating because the UI is blocked
        // when running freeze()

        // freeze();

        // this method creates a new thread for each click on the button.
        //
        // each time the button is clicked we create a separate thread
        // which delays but you can still click on the button.
        // might be an overkill but see it as an example, not programming practice
        //
        //
        nonBlocking();
    }

    private void freeze() {

        long endTime = System.currentTimeMillis() + 15 * 1000;

        while (System.currentTimeMillis() < endTime) {
            synchronized (this) {
                try {
                    wait(endTime -
                            System.currentTimeMillis());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

    }

    private void nonBlocking() {
        final Runnable runnable = new Runnable() {
            public void run() {

                long endTime = System.currentTimeMillis() + 10 * 1000;

                while (System.currentTimeMillis() < endTime) {
                    synchronized (this) {
                        try {
                            wait(endTime -
                                    System.currentTimeMillis());
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
                 Log.d(TAG, "Thread dead: Name " + Thread.currentThread().getName()
                        + " ID " + Thread.currentThread().getId()
                        + " State " + Thread.currentThread().getState());

            }
        };
        Thread mythread = new Thread(runnable);
        mythread.start();
        Log.d(TAG, "mythread Name " + mythread.getName()
                + " ID " + mythread.getId()
                + " State " + mythread.getState());
    }
}
